# Hi, man! and its versions are (c) 2019 by pX <@havok.org> (in #pX on EFNet)
# Hi, man! was written under GPL3 license.  http://gnu.org/licenses
# The gist of the licnse: use and/or modify at your own risk!
#
# Hi, man!  What have they been saying about you?
# Log highlights to a private buffer while /away (or not!)
# Hi, man! relies on words in weechat.look.highlight
# https://github.com/px-havok/weechat-himan
#
# I wrote this because my good pal narcolept wanted a highmon.pl
# that looked good with fUrlbuf (https://github.com/px-havok/weechat-furlbuf)
#
# Hi narcolept!
#
# History:
#   04.09.2019:
#       v0.1: Initial release, py3-ok
#   04.11.2019:
#           : Added 'notify' as an option
#           : Added hook_config so don't have to reload script for changes.

try:
    import weechat as w
except Exception:
    print('WeeChat (https://weechat.org/) required.')
    quit()


SCRIPT_NAME     = 'himan'
SCRIPT_AUTHOR   = 'pX @ havok'
SCRIPT_VERSION  = '0.1'
SCRIPT_LICENSE  = 'GPL3'
SCRIPT_DESC     = "What have they been saying about you?"

OPTIONS         = {'buffer_color'        : ("gray", 'color of buffer name'),
                   'nick_color'          : ("gray", 'color of mentioners nick'),
                   'notify'              : ("off", 'highlight (notify) buffer if written to'),
                   'only_away'           : ("on", 'only log highlights while /away'),
                   'outp_left'           : ("<", 'character(s) left of nick'),
                   'outp_left_color'     : ("gray", 'color of character(s) left of nick'),
                   'outp_right'          : (">", 'character(s) right of buffer name'),
                   'outp_right_color'    : ("gray", 'color of character(s) right of nick'),
                   'outp_sep'            : (" / ", 'nick/buffer separator(s)'),
                   'outp_sep_color'      : ("gray", 'color of <nick> / buffer separator(s)'),
                  }

global rst
rst = w.color('reset')


# ================================[ item ]===============================
def c(color):
    return w.color(color)


def cg(option):
    return w.config_get_plugin(option)


def himan_buffer_create():

    global himan_buffer
    himan_buffer = w.buffer_new('himan', 'himan_input_cb', '', '', '')
    w.buffer_set(himan_buffer, 'title', '-[Hi, man! v' + SCRIPT_VERSION + ']- ' + SCRIPT_DESC)
    w.buffer_set(himan_buffer, 'nicklist', '0')

    # configurable option to set buffer notify on or off
    w.buffer_set(himan_buffer, 'notify', '0')
    if cg('notify') == 'on':
        w.buffer_set(himan_buffer, 'notify', '1')


def checker(data, buffer, date, tags, displayed, highlight, prefix, message):

    # Do nothing if no highlight words set
    if w.config_get('weechat.look.highlight') == '':
        return w.WEECHAT_RC_OK

    # if away logging is on but you're not away, do nothing
    if cg('only_away') == 'on' and not w.buffer_get_string(buffer, 'localvar_away'):
        return w.WEECHAT_RC_OK

    if int(highlight):

        tags = tags.split(',')
        nick = ''
        for idx in range(len(tags)):
            if 'nick_' in tags[idx]:
                nick = c(cg('nick_color')) + tags[idx][5:] + rst

        outp_left = c(cg('outp_left_color')) + cg('outp_left') + rst
        outp_right = c(cg('outp_right_color')) + cg('outp_right') + rst
        outp_sep = c(cg('outp_sep_color')) + cg('outp_sep') + rst
        buffername = c(cg('buffer_color')) + w.buffer_get_string(buffer, 'short_name') + rst
        sp = ' '
        # account for ACTION (/me)
        if '*' in prefix:
            sp = ' * '

        if not w.buffer_search('python', 'himan'):
            himan_buffer_create()

        w.prnt(himan_buffer, outp_left + nick + outp_sep + buffername + outp_right + sp + message)

    return w.WEECHAT_RC_OK


# ===================[ weechat options & description ]===================
def init_options():
    for option,value in list(OPTIONS.items()):
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = w.config_get_plugin(option)
        w.config_set_desc_plugin(option,'%s (default: "%s")' % (value[1], value[0]))


# dummy input bar, does nothing.
def himan_input_cb(data, buffer, input_data):
    return w.WEECHAT_RC_OK


def timer_cb(data, remaining_calls):
    w.prnt(w.current_buffer(), '%s' % data)
    return w.WEECHAT_RC_OK


# if notify option changes, update without reloading
def notify_cb(data, option, value):
    option = cg('notify')
    if option == 'on':
        w.buffer_set(himan_buffer, 'notify', '1')
    elif option == 'off':
        w.buffer_set(himan_buffer, 'notify', '0')
    return w.WEECHAT_RC_OK


def shutdown_cb():
    global himan_buffer
    himan_buffer = None
    return w.WEECHAT_RC_OK


# ================================[ main ]===============================
if __name__ == '__main__':
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, 'shutdown_cb', ''):

        init_options()

        himan_buffer_create()

        w.hook_timer(2000, 0, 1, 'timer_cb', '[himan]\tHi, man!  What are they saying about you?\n'
                            '[himan]\tHighlights will be logged to "himan" buffer\n'
                            '[himan]\tOptions: /fset himan')

        w.hook_config("plugins.var.python." + SCRIPT_NAME + ".notify", "notify_cb", "")

        w.hook_print('', 'notify_message', '', 0, 'checker', '')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
#
#   Based on go.py by FlashCode
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Set screen title
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2019-07-11, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.5: make script compatible with Python 3
# 2019-07-11, Simmo Saan <simmo.saan@gmail.com>
#     version 0.4: fix detection of "/input search_text_here"
# 2016-02-20, Simmo Saan <simmo.saan@gmail.com>
#     version 0.3: add option to only display selected command
# 2010-01-19, xt <xt@bash.no>
#     version 0.2: return max 9 commands
# 2009-06-10, xt <xt@bash.no>
#     version 0.1: initial release

import weechat as w
import re
weechat = w

SCRIPT_NAME    = "histsearch"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.5"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Quick search in command history (think ctrl-r in bash)"
SCRIPT_COMMAND = 'histsearch'

# script options
settings = {
    "color_number"                 : "yellow,magenta",
    "color_number_selected"        : "yellow,red",
    "color_name"                   : "black,cyan",
    "color_name_selected"          : "black,brown",
    "color_name_highlight"         : "red,cyan",
    "color_name_highlight_selected": "red,brown",
    "message"                      : "Command: ",
    "display_selected_only"        : "false",
}


# hooks management
hook_command_run = {
    "input" : ("/input *",  "command_run_input"),
    "buffer": ("/buffer *", "command_run_buffer"),
    "window": ("/window *", "command_run_window"),
}
hooks = {}

# input before command (we'll restore it later)
saved_input = ""

# last user input (if changed, we'll update list of matching commands)
old_input = None

# matching buffers
commands = []
command_pos = 0

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    weechat.hook_command(SCRIPT_COMMAND, "Quick search in command history", "",
                         "You can bind command to a key, for example:\n  /key bind meta-e /histsearch\n\n" +
                         "You can use completion key (commonly Tab and shift-Tab) to select " +
                         "next/previous command in list.",
                         "", "histsearch_cmd", "")
    for option, default_value in settings.items():
        if w.config_get_plugin(option) == "":
            w.config_set_plugin(option, default_value)


def unhook_one(hook):
    """ Unhook something hooked by this script """
    global hooks
    if hook in hooks:
        w.unhook(hooks[hook])
        del hooks[hook]

def unhook_all():
    """ Unhook all """
    global hook_command_run
    unhook_one("modifier")
    map(unhook_one, hook_command_run.keys())
    return w.WEECHAT_RC_OK

def hook_all():
    """ Hook command_run and modifier """
    global hook_command_run, hooks
    for hook, value in hook_command_run.items():
        if hook not in hooks:
            hooks[hook] = w.hook_command_run(value[0], value[1], "")
    if "modifier" not in hooks:
        hooks["modifier"] = w.hook_modifier(
            "input_text_display_with_cursor", "input_modifier", "")

def histsearch_start(buffer):
    """ Start histsearch on buffer """
    global saved_input, old_input, commands_pos
    hook_all()
    saved_input = w.buffer_get_string(buffer, "input")
    w.buffer_set(buffer, "input", "")
    old_input = None
    commands_pos = 0

def histsearch_end(buffer):
    """ End histsearch on buffer """
    global saved_input, old_input
    unhook_all()
    w.buffer_set(buffer, "input", saved_input)
    old_input = None

def get_matching_commands(input):
    """ Return list with commands matching user input """
    global commands_pos
    clist = []
    if len(input) == 0:
        commands_pos = 0
        return []
    input = input.lower()
    infolist = w.infolist_get("history", "", "")
    while w.infolist_next(infolist):
        text = w.infolist_string(infolist, "text")
        matching = input in text
        #if not matching and input.isdigit():
        #    matching = str(number).startswith(input)
        if len(clist) > 9:
           # Return max 10 commands
            break
        if len(input) == 0 or matching:
            if not text in clist:
                clist.append(text)
    w.infolist_free(infolist)
    return clist

def get_command_string(commands, pos, input):
    ''' Build the string that is displayed on input bar '''

    global settings

    colors = {}
    returnstr = ''

    for option in settings:
        colors[option] = weechat.config_get_plugin(option)

    for i, command in enumerate(commands):
        selected = ''
        if i == pos:
            selected = "_selected"

        if weechat.config_string_to_boolean(weechat.config_get_plugin("display_selected_only")):
            if not selected:
                continue

        index = command.find(input)
        index2 = index + len(input)
        returnstr += ' ' + \
                weechat.color(colors["color_number" + selected]) + str(i) + \
                weechat.color(colors["color_name" + selected]) + command[:index] + \
                weechat.color(colors["color_name_highlight" + selected]) + command[index:index2] + \
                weechat.color(colors["color_name" + selected]) + command[index2:] + \
                weechat.color("reset")

        if i > 10:
            # Display max 10 commands
            break

    return returnstr

def input_modifier(data, modifier, modifier_data, string):
    """ This modifier is called when input text item is built by WeeChat
    (commonly after changes in input or cursor move), it builds new input with
    prefix ("Commands:"), and suffix (list of commands found) """
    global old_input, commands, commands_pos
    if modifier_data != w.current_buffer():
        return ""
    input = w.string_remove_color(string, "")
    input = input.strip()
    if old_input == None or input != old_input:
        old_commands = commands
        commands = get_matching_commands(input)
        if commands != old_commands and len(input) > 0:
            commands_pos = 0
        old_input = input
    commandstr = get_command_string(commands, commands_pos, input)
    return w.config_get_plugin("message") + string + commandstr

def command_run_input(data, buffer, command):
    """ Function called when a command "/input xxxx" is run """
    global commands, commands_pos
    if command.startswith('/input search_text') or command.startswith('/input jump'):
        # search text or jump to another buffer is forbidden now
        return w.WEECHAT_RC_OK_EAT
    elif command == "/input complete_next":
        # choose next buffer in list
        commands_pos += 1
        if commands_pos >= len(commands):
            commands_pos = 0
        w.hook_signal_send("input_text_changed",
                                 w.WEECHAT_HOOK_SIGNAL_STRING, "")
        return w.WEECHAT_RC_OK_EAT
    elif command == "/input complete_previous":
        # choose previous buffer in list
        commands_pos -= 1
        if commands_pos < 0:
            commands_pos = len(commands) - 1
        w.hook_signal_send("input_text_changed",
                                 w.WEECHAT_HOOK_SIGNAL_STRING, "")
        return w.WEECHAT_RC_OK_EAT
    elif command == "/input return":
        # As in enter was pressed.
        # Put the current command on the input bar
        histsearch_end(buffer)
        if len(commands) > 0:
            w.command(buffer, "/input insert " + commands[commands_pos])
        return w.WEECHAT_RC_OK_EAT
    return w.WEECHAT_RC_OK

def command_run_buffer(data, buffer, command):
    """ Function called when a command "/buffer xxxx" is run """
    return w.WEECHAT_RC_OK_EAT

def command_run_window(data, buffer, command):
    """ Function called when a command "/buffer xxxx" is run """
    return w.WEECHAT_RC_OK_EAT

def histsearch_cmd(data, buffer, args):
    """ Command "/histsearch": just hook what we need """
    global hooks

    if "modifier" in hooks:
        histsearch_end(buffer)
    else:
        histsearch_start(buffer)
    return w.WEECHAT_RC_OK

def histsearch_unload_script():
    """ Function called when script is unloaded """
    unhook_all()
    return w.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 by Simmo Saan <simmo.saan@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# History:
#
# 2019-06-07, Trygve Aaberge <trygveaa@gmail.com>
#   version 0.10: remove newlines from command completion
# 2015-10-04, Simmo Saan <simmo.saan@gmail.com>
#   version 0.9: fix text search imitation in filter
# 2015-08-27, Simmo Saan <simmo.saan@gmail.com>
#   version 0.8: add documentation
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.7: mute filter add/del
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.6: imitate search settings in filter
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.5: option for bar item text
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.4: option for default state
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.3: allow toggling during search
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.2: add bar item for indication
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#   version 0.1: initial script
#

"""
Filter buffers automatically while searching them
"""

from __future__ import print_function

SCRIPT_NAME = "grep_filter"
SCRIPT_AUTHOR = "Simmo Saan <simmo.saan@gmail.com>"
SCRIPT_VERSION = "0.10"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Filter buffers automatically while searching them"

SCRIPT_REPO = "https://github.com/sim642/grep_filter"

SCRIPT_COMMAND = SCRIPT_NAME
SCRIPT_BAR_ITEM = SCRIPT_NAME
SCRIPT_LOCALVAR = SCRIPT_NAME

IMPORT_OK = True

try:
	import weechat
except ImportError:
	print("This script must be run under WeeChat.")
	print("Get WeeChat now at: http://www.weechat.org/")
	IMPORT_OK = False

import re # re.escape

SETTINGS = {
	"enable": (
		"off",
		"enable automatically start filtering when searching"),
	"bar_item": (
		"grep",
		"text to show in bar item when filtering")
}

KEYS = {
	"ctrl-G": "/%s toggle" % SCRIPT_COMMAND
}

def get_merged_buffers(ptr):
	"""
	Get a list of buffers which are merged with "ptr".
	"""

	hdata = weechat.hdata_get("buffer")
	buffers = weechat.hdata_get_list(hdata, "gui_buffers")
	buffer = weechat.hdata_search(hdata, buffers, "${buffer.number} == %i" % weechat.hdata_integer(hdata, ptr, "number"), 1)
	nbuffer = weechat.hdata_move(hdata, buffer, 1)

	ret = []
	while buffer:
		ret.append(weechat.hdata_string(hdata, buffer, "full_name"))

		if (weechat.hdata_integer(hdata, buffer, "number") == weechat.hdata_integer(hdata, nbuffer, "number")):
			buffer = nbuffer
			nbuffer = weechat.hdata_move(hdata, nbuffer, 1)
		else:
			buffer = None

	return ret

def filter_exists(name):
	"""
	Check whether a filter named "name" exists.
	"""

	hdata = weechat.hdata_get("filter")
	filters = weechat.hdata_get_list(hdata, "gui_filters")
	filter = weechat.hdata_search(hdata, filters, "${filter.name} == %s" % name, 1)

	return bool(filter)

def filter_del(name):
	"""
	Delete a filter named "name".
	"""

	weechat.command(weechat.buffer_search_main(), "/mute filter del %s" % name)

def filter_addreplace(name, buffers, tags, regex):
	"""
	Add (or replace if already exists) a filter named "name" with specified argumets.
	"""

	if filter_exists(name):
		filter_del(name)

	weechat.command(weechat.buffer_search_main(), "/mute filter add %s %s %s %s" % (name, buffers, tags, regex))

def buffer_searching(buffer):
	"""
	Check whether "buffer" is in search mode.
	"""

	hdata = weechat.hdata_get("buffer")

	return bool(weechat.hdata_integer(hdata, buffer, "text_search"))

def buffer_filtering(buffer):
	"""
	Check whether "buffer" should be filtered.
	"""

	local = weechat.buffer_get_string(buffer, "localvar_%s" % SCRIPT_LOCALVAR)
	return {"": None, "0": False, "1": True}[local]

def buffer_build_regex(buffer):
	"""
	Build a regex according to "buffer"'s search settings.
	"""

	hdata = weechat.hdata_get("buffer")
	input = weechat.hdata_string(hdata, buffer, "input_buffer")
	exact = weechat.hdata_integer(hdata, buffer, "text_search_exact")
	where = weechat.hdata_integer(hdata, buffer, "text_search_where")
	regex = weechat.hdata_integer(hdata, buffer, "text_search_regex")

	if not regex:
		input = re.escape(input)

	if exact:
		input = "(?-i)%s" % input

	filter_regex = None
	if where == 1: # message
		filter_regex = input
	elif where == 2: # prefix
		filter_regex = "%s\\t" % input
	else: # prefix | message
		filter_regex = input # TODO: impossible with current filter regex

	return "!%s" % filter_regex

def buffer_update(buffer):
	"""
	Refresh filtering in "buffer" by updating (or removing) the filter and update the bar item.
	"""

	hdata = weechat.hdata_get("buffer")

	buffers = ",".join(get_merged_buffers(buffer))
	name = "%s_%s" % (SCRIPT_NAME, buffers)

	if buffer_searching(buffer):
		if buffer_filtering(buffer):
			filter_addreplace(name, buffers, "*", buffer_build_regex(buffer))
		elif not buffer_filtering(buffer) and filter_exists(name):
			filter_del(name)
	elif filter_exists(name):
		filter_del(name)

	where = weechat.hdata_integer(hdata, buffer, "text_search_where")
	weechat.buffer_set(buffer, "localvar_set_%s_warn" % SCRIPT_LOCALVAR, "1" if where == 3 else "0") # warn about incorrect filter

	weechat.bar_item_update(SCRIPT_BAR_ITEM)

def input_search_cb(data, signal, buffer):
	"""
	Handle "input_search" signal.
	"""

	if buffer_searching(buffer) and buffer_filtering(buffer) is None:
		enable = weechat.config_string_to_boolean(weechat.config_get_plugin("enable"))
		weechat.buffer_set(buffer, "localvar_set_%s" % SCRIPT_LOCALVAR, "1" if enable else "0")
		weechat.buffer_set(buffer, "localvar_set_%s_warn" % SCRIPT_LOCALVAR, "0")
	elif not buffer_searching(buffer):
		weechat.buffer_set(buffer, "localvar_del_%s" % SCRIPT_LOCALVAR, "")
		weechat.buffer_set(buffer, "localvar_del_%s_warn" % SCRIPT_LOCALVAR, "")

	buffer_update(buffer)

	return weechat.WEECHAT_RC_OK

def input_text_changed_cb(data, signal, buffer):
	"""
	Handle "input_text_changed" signal.
	"""

	if buffer_searching(buffer) and buffer_filtering(buffer):
		buffers = ",".join(get_merged_buffers(buffer))
		name = "%s_%s" % (SCRIPT_NAME, buffers)

		filter_addreplace(name, buffers, "*", buffer_build_regex(buffer))

	return weechat.WEECHAT_RC_OK

def command_cb(data, buffer, args):
	"""
	Handle command.
	"""

	if args == "enable":
		weechat.buffer_set(buffer, "localvar_set_%s" % SCRIPT_LOCALVAR, "1")
	elif args == "disable":
		weechat.buffer_set(buffer, "localvar_set_%s" % SCRIPT_LOCALVAR, "0")
	elif args == "toggle":
		weechat.buffer_set(buffer, "localvar_set_%s" % SCRIPT_LOCALVAR, "0" if buffer_filtering(buffer) else "1")
	else:
		pass

	buffer_update(buffer)

	return weechat.WEECHAT_RC_OK

def bar_item_cb(data, item, window, buffer, extra_info):
	"""
	Build the bar item's content for "buffer".
	"""

	buffers = ",".join(get_merged_buffers(buffer))
	name = "%s_%s" % (SCRIPT_NAME, buffers)

	if filter_exists(name):
		warn = int(weechat.buffer_get_string(buffer, "localvar_%s_warn" % SCRIPT_LOCALVAR))

		return "%s%s%s" % (
			weechat.color("input_text_not_found" if warn else "bar_fg"),
			weechat.config_get_plugin("bar_item"),
			weechat.color("reset"))
	else:
		return ""

if __name__ == "__main__" and IMPORT_OK:
	if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
		weechat.hook_signal("input_search", "input_search_cb", "")
		weechat.hook_signal("input_text_changed", "input_text_changed_cb", "")

		weechat.hook_command(SCRIPT_COMMAND, SCRIPT_DESC,
"""enable || disable || toggle""",
""" enable: enable {0} in current buffer
disable: disable {0} in current buffer
 toggle: toggle {0} in current buffer

By default a bind in "search" context is added to toggle with "ctrl-G".
To see {0} status during search, add "{1}" item to some bar. On default configuration you can do it with:
    /set weechat.bar.input.items "[input_prompt]+(away),[{1}],[input_search],[input_paste],input_text"

Due to technical reasons with /filter it is not possible to exactly {0} in "pre|msg" search mode, thus the bar item is shown in warning color.""".format(SCRIPT_NAME, SCRIPT_BAR_ITEM),
"""enable || disable || toggle""",
		"command_cb", "")

		weechat.bar_item_new("(extra)%s" % SCRIPT_BAR_ITEM, "bar_item_cb", "")

		for option, value in SETTINGS.items():
			if not weechat.config_is_set_plugin(option):
				weechat.config_set_plugin(option, value[0])

			weechat.config_set_desc_plugin(option, "%s (default: \"%s\")" % (value[1], value[0]))

		weechat.key_bind("search", KEYS)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2009-06-18, xt <xt@bash.no>
#     version 0.1: initial release
#
# 2009-10-18, LBo <leon@tim-online.nl>
#     version 0.2: added autosaving of join channels
#     /set plugins.var.python.autojoin.autosave 'on'
#
# 2009-10-19, LBo <leon@tim-online.nl>
#     version 0.2.1: now only responds to part messages from self
#     find_channels() only returns join'ed channels, not all the buffers
#     updated description and docs
#
# 2009-10-20, LBo <leon@tim-online.nl>
#     version 0.2.2: fixed quit callback
#     removed the callbacks on part & join messages
#
# 2012-04-14, Filip H.F. "FiXato" Slagter <fixato+weechat+autojoin@gmail.com>
#     version 0.2.3: fixed bug with buffers of which short names were changed.
#                    Now using 'name' from irc_channel infolist.
#     version 0.2.4: Added support for key-protected channels
#
# 2014-05-22, Nathaniel Wesley Filardo <PADEBR2M2JIQN02N9OO5JM0CTN8K689P@cmx.ietfng.org>
#     version 0.2.5: Fix keyed channel support
#
# 2016-01-13, The fox in the shell <KellerFuchs@hashbang.sh>
#     version 0.2.6: Support keeping chan list as secured data
#
# 2018-08-09, Julien Palard <julien@palard.fr>
#     version 0.3.0: Support for Python 3
#
# 2019-09-28, fructose
#     version 0.3.1: Error on invalid arguments
#
# @TODO: add options to ignore certain buffers
# @TODO: maybe add an option to enable autosaving on part/join messages

import weechat as w
import re

SCRIPT_NAME    = "autojoin"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.3.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Configure autojoin for all servers according to currently joined channels"
SCRIPT_COMMAND = "autojoin"

# script options
settings = {
    "autosave": "off",
}

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):

    w.hook_command(SCRIPT_COMMAND,
                   SCRIPT_DESC,
                   "[--run]",
                   "   --run: actually run the commands instead of displaying\n",
                   "--run",
                   "autojoin_cb",
                   "")

    # w.hook_signal('*,irc_in2_join', 'autosave_channels_on_activity', '')
    # w.hook_signal('*,irc_in2_part', 'autosave_channels_on_activity', '')
    w.hook_signal('quit',           'autosave_channels_on_quit', '')

# Init everything
for option, default_value in settings.items():
    if w.config_get_plugin(option) == "":
        w.config_set_plugin(option, default_value)


def autosave_channels_on_quit(signal, callback, callback_data):
    ''' Autojoin current channels '''
    if w.config_get_plugin(option) != "on":
        return w.WEECHAT_RC_OK

    items = find_channels()

    # print/execute commands
    for server, channels in items.items():
        process_server(server, channels)

    return w.WEECHAT_RC_OK


def autosave_channels_on_activity(signal, callback, callback_data):
    ''' Autojoin current channels '''
    if w.config_get_plugin(option) != "on":
        return w.WEECHAT_RC_OK

    items = find_channels()

    # print/execute commands
    for server, channels in items.items():
        nick = w.info_get('irc_nick', server)

        pattern = "^:%s!.*(JOIN|PART) :?(#[^ ]*)( :.*$)?" % nick
        match = re.match(pattern, callback_data)

        if match:  # check if nick is my nick. In that case: save
            process_server(server, channels)
        else:  # someone else: ignore
            continue

    return w.WEECHAT_RC_OK


def autojoin_cb(data, buffer, args):
    if args == '--run':
        run = True
    elif args != '':
        w.prnt('', 'Unexpected argument: %s' % args)
        return w.WEECHAT_RC_ERROR
    else:
        run = False

    # Old behaviour: doesn't save empty channel list
    # In fact should also save open buffers with a /part'ed channel
    # But I can't believe somebody would want that behaviour
    items = find_channels()

    # print/execute commands
    for server, channels in items.items():
        process_server(server, channels, run)

    return w.WEECHAT_RC_OK


def process_server(server, channels, run=True):
    option = "irc.server.%s.autojoin" % server
    channels = channels.rstrip(',')
    oldchans = w.config_string(w.config_get(option))

    if not channels:  # empty channel list
        return

    # Note: re already caches the result of regexp compilation
    sec = re.match('^\${sec\.data\.(.*)}$', oldchans)
    if sec:
        secvar = sec.group(1)
        command = "/secure set %s %s" % (secvar, channels)
    else:
        command = "/set irc.server.%s.autojoin '%s'" % (server, channels)

    if run:
        w.command('', command)
    else:
        w.prnt('', command)


def find_channels():
    """Return list of servers and channels"""
    # TODO: make it return a dict with more options like "nicks_count etc."
    items = {}
    infolist = w.infolist_get('irc_server', '', '')
    # populate servers
    while w.infolist_next(infolist):
        items[w.infolist_string(infolist, 'name')] = ''

    w.infolist_free(infolist)

    # populate channels per server
    for server in items.keys():
        keys = []
        keyed_channels = []
        unkeyed_channels = []
        items[server] = ''  # init if connected but no channels
        infolist = w.infolist_get('irc_channel', '',  server)
        while w.infolist_next(infolist):
            if w.infolist_integer(infolist, 'nicks_count') == 0:
                # parted but still open in a buffer: bit hackish
                continue
            if w.infolist_integer(infolist, 'type') == 0:
                key = w.infolist_string(infolist, "key")
                if len(key) > 0:
                    keys.append(key)
                    keyed_channels.append(w.infolist_string(infolist, "name"))
                else:
                    unkeyed_channels.append(w.infolist_string(infolist, "name"))
        items[server] = ','.join(keyed_channels + unkeyed_channels)
        if len(keys) > 0:
            items[server] += ' %s' % ','.join(keys)
        w.infolist_free(infolist)

    return items



#!/usr/bin/env python
# coding: utf-8
#
# Copyright (c) 2012 by nesthib <nesthib@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# This script allows to create a nick list from pattern to highlight
# a bunch of nick in a channel
#
# 2012-01-15: nesthib <nesthib@gmail.com>
#        0.1: initial release

try:
    import weechat as w
except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

import argparse, re

name = "hl_nicks"
author = "nesthib <nesthib@gmail.com>"
version = "0.1.1"
license = "GPL"
description = "Generates a list of nicks in input by selecting nicks using flags and patterns"
shutdown_function = ""
charset = ""

w.register(name, author, version, license, description, shutdown_function, charset)

buffer = w.current_buffer()

settings = {
        'ignore_list': 'ChanServ,.*bot.*',
        'ignore_self': 'on',
        'separator'  : ', ',
        'short_regex': 'on',
        'sort_nicks' : 'on',
        'ignore_case': 'on',
        }

for opt, val in settings.iteritems():
    if not w.config_is_set_plugin(opt):
        w.config_set_plugin(opt, val)

def get_nicklist(server, channel):
    global options

    regex_flags = 0
    if options['ignore_case']:
        regex_flags = re.IGNORECASE
    ignore_list = w.config_get_plugin('ignore_list')
    if ignore_list == '':
        ignore_match = lambda x: False
    else:
        ignore_match = re.compile('(%s)$' % ignore_list.replace(',', '|'), regex_flags).match

    server = w.buffer_get_string(w.current_buffer(), 'localvar_server')
    my_nick = w.info_get('irc_nick', server)
    nicklist = {}
    infolist_nicklist = w.infolist_get('nicklist', w.current_buffer(), '')
    while w.infolist_next(infolist_nicklist):
        nick = w.infolist_string(infolist_nicklist, 'name')
        prefix = w.infolist_string(infolist_nicklist, 'prefix')
        nick_type = w.infolist_string(infolist_nicklist, 'type')
        if nick_type != 'nick' or (options['ignore_self'] and nick == my_nick) or ignore_match(nick):
            pass
        else:
            if not nicklist.has_key(prefix):
                nicklist[prefix]=[]
            nicklist[prefix].append(nick)
    w.infolist_free(infolist_nicklist)
    return nicklist

def my_hl_cb(data, buffer, args):

    flags_relations = { '~' : 'q',
                        '&' : 'a',
                        '@' : 'o',
                        '%' : 'h',
                        '+' : 'v',
                        ' ' : 'n' }

    parser = argparse.ArgumentParser(prefix_chars='-+', add_help=False)
    for flag in flags_relations.values():
        parser.add_argument('+%s' % flag, action='store_const', const=True)
        parser.add_argument('-%s' % flag, action='store_const', const=False)

    try:
        (opts, args) = parser.parse_known_args(args.split())
    except SystemExit:
        w.prnt('', 'Error: in "%s", invalid options. See /help %s for authorized options.' % (args, name))
        return w.WEECHAT_RC_ERROR

    opts = vars(opts)
    used_opts = list(set(opts.values()))

    channel = w.buffer_get_string(buffer, 'localvar_channel')
    server = w.buffer_get_string(buffer, 'localvar_server')

    nickgroups = get_nicklist(server, channel)
    nicks = []
    invert_match = False

    if args:
        regex = args[0]
        if regex.startswith('!'):
            invert_match = True
            regex = regex[1:]
        regex_flags = 0
        if options['ignore_case']:
            regex_flags = re.IGNORECASE
        try:
            if options['short_regex']:
                regex = re.compile('.*%s.*' % regex, regex_flags)
            else:
                regex = re.compile(regex, regex_flags)
        except:
            w.prnt('', 'Error with argument "%s" invalid regexp' % args[0])
            regex = None
            invert_match = False

    for flag in reversed(sorted(nickgroups)):
        if not flags_relations.has_key(flag):
            w.prnt('', 'Error: flag "%s" is not supported' % flag)
            continue
        if used_opts == [None]:
            pass
        elif True in used_opts and False in used_opts:
            w.prnt('', 'Error: + and - options are not compatible')
            return w.WEECHAT_RC_OK
        elif True in used_opts:
            if opts[flags_relations[flag]]:
                pass
            else:
                continue
        elif False in used_opts and opts[flags_relations[flag]] ==  False:
            continue
        for nick in nickgroups[flag]:
            if not args or (regex and (bool(regex.match(nick)) ^ invert_match)):
                nicks.append(nick)

    if options['sort_nicks']:
        if options['ignore_case']:
            nicks.sort(key=str.lower)
        else:
            nicks.sort()
    separator = w.config_get_plugin('separator')
    trailing_char = w.config_string(w.config_get('weechat.completion.nick_completer'))
    input_text = separator.join(nicks)
    if input_text:
        input_text = input_text+trailing_char
    w.buffer_set(w.current_buffer(), 'input', input_text)
    w.command ("", "/input move_end_of_line")

    return w.WEECHAT_RC_OK

invertdict = lambda d: dict(zip(d.itervalues(), d.keys()))
booleans = {'on': True, 'off': False}
boolean_options = ['ignore_self', 'short_regex', 'sort_nicks', 'ignore_case']

options = {}
for option in settings.keys():
    if option in boolean_options :
        options[option] = booleans[w.config_get_plugin(option)]
    else:
        options[option] = w.config_get_plugin(option)

def my_config_cb(data, option, value):
    global options

    for boolean_option in boolean_options :
        if option.endswith(boolean_option):
            if value in booleans.keys():
                options[boolean_option] = booleans[w.config_get_plugin(boolean_option)]
            else:
                w.prnt('', 'Error: "%s" is not a boolean, please use "on" or "off"' % w.config_get_plugin(boolean_option))
                w.config_set_plugin(boolean_option, invertdict(booleans)[options[boolean_option]])

    return w.WEECHAT_RC_OK

for option in settings.keys():
    w.hook_config("plugins.var.python.%s.%s" % (name, option), "my_config_cb", "")

w.hook_command("hl", description, "",
"""    usage: /hl [+/-ovn] [regex]

        +o, +v, +n  : add opped, voiced, normal nicks to highlighted nicks
        -o, -v, -n  : remove opped, voiced, normal nicks from highlighted nicks
        regex       : select nicks based on regex

        alternate flags owner (q), admin (a) and halfop (h) can be used on networks supporting them

        EXAMPLES
        /hl +ov     : highlight opped and voices users
        /hl -o      : highlight everyone except opped users
        /hl ^n      : highlight nicks starting with "n"
        /hl !bot    : do not highlight nicks comprising "bot"

        OPTIONS
        ignore_list : comma separated list of nicks to ignore
        ignore_self : boolean option to trigger addition of own nick to list
        separator   : string used as separator for list of nicks
        short_regex : boolean option to replace regex by .*regex.*
        sort_nicks  : boolean option to sort nicks alphabetically (otherwise by group)
        ignore_case : boolean option to perfom case insensitive nick matches
""", "", "my_hl_cb", "")




# -*- coding: utf-8 -*-
#
# Project:     weechat-tts
# Homepage:    https://git.hashtagueule.fr/raspbeguy/weechat-tts
# Description: Text-to-speech script.
#              Requires a TTS engine such as espeak or picospeaker.
# License:     MIT (see below)
#
# Copyright (c) 2017 by raspbeguy
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import subprocess
import sys
import time


# Ensure that we are running under WeeChat.
try:
    import weechat
except ImportError:
    print('This script has to run under WeeChat (https://weechat.org/).')
    sys.exit(1)


# Name of the script.
SCRIPT_NAME = 'tts'

# Author of the script.
SCRIPT_AUTHOR = 'raspbeguy'

# Version of the script.
SCRIPT_VERSION = '0.2.1'

# License under which the script is distributed.
SCRIPT_LICENSE = 'MIT'

# Description of the script.
SCRIPT_DESC = 'Uses a TTS engine to pronounce messages'

# Name of a function to be called when the script is unloaded.
SCRIPT_SHUTDOWN_FUNC = ''

# Used character set (utf-8 by default).
SCRIPT_CHARSET = ''

# Script options.
OPTIONS = {
    'tts_engine': (
        'espeak',
        'TTS engine to use.'
        'Valid values: espeak, festival, picospeaker'
    ),
    'language': (
        '',
        'Language used by the TTS engine'
    ),
    'tts_on_highlights': (
        'on',
        'Pronounce on highlights.'
    ),
    'tts_on_privmsgs': (
        'on',
        'Pronounce private messages.'
    ),
    'tts_on_filtered_messages': (
        'off',
        'Pronounce also filtered (hidden) messages.'
    ),
    'tts_when_away': (
        'on',
        'Pronounce also when away.'
    ),
    'tts_for_current_buffer': (
        'on',
        'Pronounce also for the currently active buffer.'
    ),
    'tts_on_all_messages_in_buffers': (
        '',
        'A comma-separated list of buffers in which you want to '
        'pronounce all messages that appear in them.'
    ),
    'min_notification_delay': (
        '500',
        'A minimal delay between successive spoken message from the same '
        'buffer (in milliseconds; set to 0 to show all notifications).'
    ),
    'ignore_messages_tagged_with': (
        # irc_join:      Joined IRC
        # irc_quit:      Quit IRC
        # irc_part:      Parted a channel
        # irc_status:    Status messages
        # irc_nick_back: A nick is back on server
        # irc_401:       No such nick/channel
        # irc_402:       No such server
        'irc_join,irc_quit,irc_part,irc_status,irc_nick_back,irc_401,irc_402',
        'A comma-separated list of message tags for which no text '
        'should be said.'
    ),
    'ignore_buffers': (
        '',
        'A comma-separated list of buffers from which no text should be said.'
    ),
    'ignore_buffers_starting_with': (
        '',
        'A comma-separated list of buffer prefixes from which no '
        'text should be said'
    ),
    'ignore_nicks': (
        '',
        'A comma-separated list of nicks from which no text should '
        'be said.'
    ),
    'ignore_nicks_starting_with': (
        '',
        'A comma-separated list of nick prefixes from which no '
        'text should be said.'
    ),
    'tts_url': (
        'off',
        "Pronounce URLs."
    ),
    'format_chan_message': (
         '%(nick)s says in %(chan)s: %(body)s.',
         'Text spoken for a message in a regular channel.'
    ),
    'format_priv_message': (
         '%(nick)s says: %(body)s.',
         'Text spoken for a message in a private conversation.'
    ),
}

def default_value_of(option):
    """Returns the default value of the given option."""
    return OPTIONS[option][0]


def add_default_value_to(description, default_value):
    """Adds the given default value to the given option description."""
    # All descriptions end with a period, so do not add another period.
    return '{} Default: {}.'.format(
        description,
        default_value if default_value else '""'
    )


def nick_that_sent_message(tags, prefix):
    """Returns a nick that sent the message based on the given data passed to
    the callback.
    """
    # 'tags' is a comma-separated list of tags that WeeChat passed to the
    # callback. It should contain a tag of the following form: nick_XYZ, where
    # XYZ is the nick that sent the message.
    for tag in tags:
        if tag.startswith('nick_'):
            return tag[5:]

    # There is no nick in the tags, so check the prefix as a fallback.
    # 'prefix' (str) is the prefix of the printed line with the message.
    # Usually (but not always), it is a nick with an optional mode (e.g. on
    # IRC, @ denotes an operator and + denotes a user with voice). We have to
    # remove the mode (if any) before returning the nick.
    # Strip also a space as some protocols (e.g. Matrix) may start prefixes
    # with a space. It probably means that the nick has no mode set.
    if prefix.startswith(('~', '&', '@', '%', '+', '-', ' ')):
        return prefix[1:]

    return prefix


def parse_tags(tags):
    """Parses the given "list" of tags (str) from WeeChat into a list.
    """
    return tags.split(',')


def message_printed_callback(data, buffer, date, tags, is_displayed,
                             is_highlight, prefix, message):
    """A callback when a message is printed."""
    is_displayed = int(is_displayed)
    is_highlight = int(is_highlight)
    tags = parse_tags(tags)
    nick = nick_that_sent_message(tags, prefix)

    if message_should_be_voiced(buffer, tags, nick, is_displayed, is_highlight):
        text = prepare_tts(buffer, nick, message)
        tts(text)

    return weechat.WEECHAT_RC_OK


def message_should_be_voiced(buffer, tags, nick, is_displayed, is_highlight):
    """Should a notification be sent?"""
    if message_should_be_voiced_disregarding_time(buffer, tags, nick,
                                                     is_displayed, is_highlight):
        # The following function should be called only when the notification
        # should be sent (it updates the last notification time).
        if not is_below_min_notification_delay(buffer):
            return True
    return False


def message_should_be_voiced_disregarding_time(buffer, tags, nick,
                                                  is_displayed, is_highlight):
    """Should a notification be sent when not considering time?"""
    if not nick:
        # A nick is required to form a correct notification source/message.
        return False

    if i_am_author_of_message(buffer, nick):
        return False

    if not is_displayed:
        if not tts_on_filtered_messages():
            return False

    if buffer == weechat.current_buffer():
        if not tts_for_current_buffer():
            return False

    if is_away(buffer):
        if not tts_when_away():
            return False

    if ignore_messages_tagged_with(tags):
        return False

    if ignore_messages_from_nick(nick):
        return False

    if ignore_messages_from_buffer(buffer):
        return False

    if is_private_message(buffer):
        return tts_on_private_messages()

    if is_highlight:
        return tts_on_highlights()

    if tts_on_all_messages_in_buffer(buffer):
        return True

    return False


def is_below_min_notification_delay(buffer):
    """Is a notification in the given buffer below the minimal delay between
    successive notifications from the same buffer?

    When called, this function updates the time of the last notification.
    """
    # We store the time of the last notification in a buffer-local variable to
    # make it persistent over the lifetime of this plugin.
    LAST_NOTIFICATION_TIME_VAR = 'last_tts_time'
    last_notification_time = buffer_get_float(
        buffer,
        'localvar_' + LAST_NOTIFICATION_TIME_VAR
    )

    min_notification_delay = weechat.config_get_plugin('min_notification_delay')
    # min_notification_delay is in milliseconds (str). To compare it with
    # last_notification_time (float in seconds), we have to convert it to
    # seconds (float).
    min_notification_delay = float(min_notification_delay) / 1000

    current_time = time.time()

    # We have to update the last notification time before returning the result.
    buffer_set_float(
        buffer,
        'localvar_set_' + LAST_NOTIFICATION_TIME_VAR,
        current_time
    )

    return (min_notification_delay > 0 and
            current_time - last_notification_time < min_notification_delay)


def buffer_get_float(buffer, property):
    """A variant of weechat.buffer_get_x() for floats.

    This variant is needed because WeeChat supports only buffer_get_string()
    and buffer_get_int().
    """
    value = weechat.buffer_get_string(buffer, property)
    return float(value) if value else 0.0


def buffer_set_float(buffer, property, value):
    """A variant of weechat.buffer_set() for floats.

    This variant is needed because WeeChat supports only integers and strings.
    """
    weechat.buffer_set(buffer, property, str(value))


def names_for_buffer(buffer):
    """Returns a list of all names for the given buffer."""
    # The 'buffer' parameter passed to our callback is actually the buffer's ID
    # (e.g. '0x2719cf0'). We have to check its name (e.g. 'freenode.#weechat')
    # and short name (e.g. '#weechat') because these are what users specify in
    # their configs.
    buffer_names = []

    full_name = weechat.buffer_get_string(buffer, 'name')
    if full_name:
        buffer_names.append(full_name)

    short_name = weechat.buffer_get_string(buffer, 'short_name')
    if short_name:
        buffer_names.append(short_name)
        # Consider >channel and #channel to be equal buffer names. The reason
        # is that the https://github.com/rawdigits/wee-slack plugin replaces
        # '#' with '>' to indicate that someone in the buffer is typing. This
        # fixes the behavior of several configuration options (e.g.
        # 'tts_on_all_messages_in_buffers') when weechat_notify_send is used
        # together with the wee_slack plugin.
        #
        # Note that this is only needed to be done for the short name. Indeed,
        # the full name always stays unchanged.
        if short_name.startswith('>'):
            buffer_names.append('#' + short_name[1:])

    return buffer_names


def tts_for_current_buffer():
    """Should we also send notifications for the current buffer?"""
    return weechat.config_get_plugin('tts_for_current_buffer') == 'on'


def tts_on_highlights():
    """Should we send notifications on highlights?"""
    return weechat.config_get_plugin('tts_on_highlights') == 'on'


def tts_on_private_messages():
    """Should we send notifications on private messages?"""
    return weechat.config_get_plugin('tts_on_privmsgs') == 'on'


def tts_on_filtered_messages():
    """Should we also send notifications for filtered (hidden) messages?"""
    return weechat.config_get_plugin('tts_on_filtered_messages') == 'on'


def tts_when_away():
    """Should we also send notifications when away?"""
    return weechat.config_get_plugin('tts_when_away') == 'on'


def is_away(buffer):
    """Is the user away?"""
    return weechat.buffer_get_string(buffer, 'localvar_away') != ''


def is_private_message(buffer):
    """Has a private message been sent?"""
    return weechat.buffer_get_string(buffer, 'localvar_type') == 'private'


def i_am_author_of_message(buffer, nick):
    """Am I (the current WeeChat user) the author of the message?"""
    return weechat.buffer_get_string(buffer, 'localvar_nick') == nick


def split_option_value(option, separator=','):
    """Splits the value of the given plugin option by the given separator and
    returns the result in a list.
    """
    values = weechat.config_get_plugin(option)
    return [value.strip() for value in values.split(separator)]

def get_format_chan_message():
    return weechat.config_get_plugin('format_chan_message')

def get_format_priv_message():
    return weechat.config_get_plugin('format_priv_message')

def ignore_messages_tagged_with(tags):
    """Should notifications be ignored for a message tagged with the given
    tags?
    """
    ignored_tags = split_option_value('ignore_messages_tagged_with')
    for ignored_tag in ignored_tags:
        for tag in tags:
            if tag == ignored_tag:
                return True
    return False


def ignore_messages_from_buffer(buffer):
    """Should notifications from the given buffer be ignored?"""
    buffer_names = names_for_buffer(buffer)

    for buffer_name in buffer_names:
        if buffer_name and buffer_name in ignored_buffers():
            return True

    for buffer_name in buffer_names:
        for prefix in ignored_buffer_prefixes():
            if prefix and buffer_name and buffer_name.startswith(prefix):
                return True

    return False


def ignored_buffers():
    """A generator of buffers from which notifications should be ignored."""
    for buffer in split_option_value('ignore_buffers'):
        yield buffer


def ignored_buffer_prefixes():
    """A generator of buffer prefixes from which notifications should be
    ignored.
    """
    for prefix in split_option_value('ignore_buffers_starting_with'):
        yield prefix


def ignore_messages_from_nick(nick):
    """Should notifications from the given nick be ignored?"""
    if nick in ignored_nicks():
        return True

    for prefix in ignored_nick_prefixes():
        if prefix and nick.startswith(prefix):
            return True

    return False


def ignored_nicks():
    """A generator of nicks from which notifications should be ignored."""
    for nick in split_option_value('ignore_nicks'):
        yield nick


def ignored_nick_prefixes():
    """A generator of nick prefixes from which notifications should be
    ignored.
    """
    for prefix in split_option_value('ignore_nicks_starting_with'):
        yield prefix


def buffers_to_tts_on_all_messages():
    """A generator of buffer names in which the user wants to be notified for
    all messages.
    """
    for buffer in split_option_value('tts_on_all_messages_in_buffers'):
        yield buffer


def tts_on_all_messages_in_buffer(buffer):
    """Does the user want to be notified for all messages in the given buffer?
    """
    buffer_names = names_for_buffer(buffer)
    for buf in buffers_to_tts_on_all_messages():
        if buf in buffer_names:
            return True
    return False

def remove_urls(body):
    import re
    return re.sub(r'https?:\/\/\S+[\ \t]', '', body)

def prepare_tts(buffer, nick, message):
    """Prepares the text to be spoken."""
    chan = ""
    if is_private_message(buffer):
        unformatted_message = get_format_priv_message()
    else:
        unformatted_message = get_format_chan_message()
        chan = weechat.buffer_get_string(buffer, 'short_name')
    if weechat.config_get_plugin('tts_url') == 'off':
        message = remove_urls(message)
    return unformatted_message % {'nick':nick, 'body': message, 'chan': chan}

def my_process_cb(data, command, return_code, out, err):
    if return_code == weechat.WEECHAT_HOOK_PROCESS_ERROR:
        weechat.prnt("", "Error with command '%s'" % command)
        return weechat.WEECHAT_RC_OK
    if return_code > 0:
        weechat.prnt("", "return_code = %d" % return_code)
    if err != "":
        weechat.prnt("", "stderr: %s" % err)
    return weechat.WEECHAT_RC_OK

def tts(text):
    """Pronounce the text"""
    engine = weechat.config_get_plugin('tts_engine')
    lang = weechat.config_get_plugin('language')
    if engine == 'espeak':
        args = {'arg1':text}
        if lang:
            args['arg2'] = '-v'
            args['arg3'] = lang
        hook = weechat.hook_process_hashtable('espeak',args,0,'my_process_cb','')
    elif engine == 'festival':
        args = {'stdin':'1', 'arg1':'festival', 'arg2':'--tts'}
        if lang:
            args['arg3'] = '--language'
            args['arg4'] = lang
        hook = weechat.hook_process_hashtable('festival',args,0,'my_process_cb','')
        weechat.hook_set(hook, "stdin", text)
        weechat.hook_set(hook, "stdin_close", "")
    elif engine == 'picospeaker':
        args = {'stdin':'1'}
        if lang:
            args['arg1'] = '-l'
            args['arg2'] = lang
        hook = weechat.hook_process_hashtable('picospeaker',args,0,'my_process_cb','')
        weechat.hook_set(hook, "stdin", text)
        weechat.hook_set(hook, "stdin_close", "")

if __name__ == '__main__':
    # Registration.
    weechat.register(
        SCRIPT_NAME,
        SCRIPT_AUTHOR,
        SCRIPT_VERSION,
        SCRIPT_LICENSE,
        SCRIPT_DESC,
        SCRIPT_SHUTDOWN_FUNC,
        SCRIPT_CHARSET
    )

    # Initialization.
    for option, (default_value, description) in OPTIONS.items():
        description = add_default_value_to(description, default_value)
        weechat.config_set_desc_plugin(option, description)
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, default_value)

    # Catch all messages on all buffers and strip colors from them before
    # passing them into the callback.
    weechat.hook_print('', '', '', 1, 'message_printed_callback', '')



# -*- coding: utf-8 -*-
"""
Ported to weechat by llua (Two & a half LoC. AWWWWWW YEA, 1337)
Contact: llua at irc://irc.freenode.net
----------------------------------------------------------------------------
xchat_mplayer by Csigaa
Prints 'NICK is playing FILENAME [MPlayer version]' action to the active channel/dialog.

This version is entirely based on procfs, so finally works on
all POSIX-compilant OS (*BSD, Solaris, Linux, etc) without calling any commands
(except for version number, but it is now optional)

NOTE FOR PASTEBIN: save the file as xchat_mplayer.py to your xchat directory


Contact: csigaa@gmail.com
License: Beer-ware (by phk) ;p
----------------------------------------------------------------------------
 "THE BEER-WARE LICENSE" (Revision 42):
 Csigaa wrote this file. As long as you retain this notice you
 can do whatever you want with this stuff. If we meet some day, and you think
 this stuff is worth it, you can buy me a beer in return
----------------------------------------------------------------------------

"""


import weechat
import os
import re

weechat.register("mplayer", "llua", "0.1", "The Beer-ware License", "Now Playing for MPlayer", "", "")

if os.name != 'posix':
    raise NotImplementedError,'non-POSIX systems are not supported'

def getFilename():
    # Get the video file name (identified by naming pattern) from procfs

    # list&walk the directory contents
    proclist = sorted(os.listdir('/proc'))
    for proc in os.listdir('/proc'):
	# if it is a directory with a name of digits only
	if os.path.isdir('/proc/'+proc) and re.match('[0-9]+',proc):
	    try:
		exe = os.readlink('/proc/'+proc+'/exe').split('/')
		# if executable name is 'mplayer'
		if exe.pop() == 'mplayer':
		    filelist = os.listdir('/proc/'+proc+'/fd')
		    # walk the list of open files
		    for file in filelist:
			try:
			    path = os.readlink('/proc/'+proc+'/fd/'+file)
			    if re.match('.*(avi|mpg|mkv|mp4|nuv|ogg|ogm|wmv|iso|img)$',path,re.I):
				# if video filename found, return
				return path.split('/').pop()
			except:
			    # if link not readable, skip
			    continue
	    except:
		# if process directory not readable, skip
		continue
    return None

def getVersion():
    try:
	import commands
	try:
	    # release version, begins with number (incl. optional rc sign with maximum 2 digits); if no match, exception occurs (empty list - 0 index is out of range)
	    ver = re.findall('^MPlayer\s\d[.]\d+\w{0,4}',commands.getoutput('mplayer'))[0]
	except:
	    # SVN version, begins with 'SVN' (revision number maximum 6 digits)
	    ver = re.findall('^MPlayer\sSVN-r\d{0,6}',commands.getoutput('mplayer'))[0]
	return ver
    except:
	# if any error occured above, return only the player name
	return 'MPlayer'

def mplayer_msg(world,world_eol,userdata):
    fn = getFilename()
    ver = getVersion()
    if type(fn) == str:
	# we've got a string for fn
	all = '/me is now watching: ' + fn + ' [' + ver + ']'
	weechat.command(weechat.current_buffer(), all)
	return 0
    else:
	# we've got None (or something went very-very wrong)
	return 1

weechat.hook_command("mplayer", "Now Watching", "", "/mplayer", "", "mplayer_msg", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2011 Peter A. Shevtsov <pshevtsov@severusweb.ru>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Set away status if slock is running
#
# History:
#
# 2011-10-14, Peter A. Shevtsov <pshevtsov@severusweb.ru>:
#     version 0.1: initial release
#

SCRIPT_NAME = "slock_away"
SCRIPT_AUTHOR = "Peter A. Shevtsov <pshevtsov@severusweb.ru>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Set away status if slock is running"

SCRIPT_COMMAND = "slock_away"

import_ok = True

try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

try:
    import subprocess
except ImportError:
    print "Missing package(s) for %s: %s" % (SCRIPT_NAME, message)
    import_ok = False

TIMER = None

settings = {
        'away_message': 'Away',
        'interval': '20',  # How often to check for inactivity (in seconds)
        'away': '0'
}


def set_back(overridable_messages):
    """Removes away status for servers
    where one of the overridable_messages is set"""
    if (weechat.config_get_plugin('away') == '0'):
        return  # No need to come back again
    serverlist = weechat.infolist_get('irc_server', '', '')
    if serverlist:
        buffers = []
        while weechat.infolist_next(serverlist):
            if (weechat.infolist_string(serverlist, 'away_message')
                    in overridable_messages):
                ptr = weechat.infolist_pointer(serverlist, 'buffer')
                if ptr:
                    buffers.append(ptr)
        weechat.infolist_free(serverlist)
        for buffer in buffers:
            weechat.command(buffer, "/away")
    weechat.config_set_plugin('away', '0')


def set_away(message, overridable_messages=[]):
    """Sets away status, but respectfully
    (so it doesn't change already set statuses"""
    if (weechat.config_get_plugin('away') == '1'):
        return  # No need to go away again
                # (this prevents some repeated messages)
    serverlist = weechat.infolist_get('irc_server', '', '')
    if serverlist:
        buffers = []
        while weechat.infolist_next(serverlist):
            if weechat.infolist_integer(serverlist, 'is_away') == 0:
                ptr = weechat.infolist_pointer(serverlist, 'buffer')
                if ptr:
                    buffers.append(ptr)
            elif (weechat.infolist_string(serverlist, 'away_message')
                    in overridable_messages):
                buffers.append(weechat.infolist_pointer(serverlist, 'buffer'))
        weechat.infolist_free(serverlist)
        for buffer in buffers:
            weechat.command(buffer, "/away %s" % message)
    weechat.config_set_plugin('away', '1')


def slock_away_cb(data, buffer, args):
    """Callback for /slock_away command"""
    response = {
            'msg': lambda status:
                        weechat.config_set_plugin('away_message', status)
    }
    if args:
        words = args.strip().partition(' ')
        if words[0] in response:
            response[words[0]](words[2])
        else:
            weechat.prnt('', "slock_away error: %s not a recognized command. "
                       "Try /help slock_away" % words[0])
    weechat.prnt('', "slock_away: away message: \"%s\"" %
                      weechat.config_get_plugin('away_message'))
    return weechat.WEECHAT_RC_OK


def auto_check(data, remaining_calls):
    """Callback from timer"""
    check()
    return weechat.WEECHAT_RC_OK


def check():
    """Check for existance of process and set away if it isn't there"""
    pidof = subprocess.Popen("pidof slock",
            shell=True, stdout=subprocess.PIPE)
    pidof.wait()
    if pidof.returncode == 0:
        set_away(weechat.config_get_plugin('away_message'), [])
    else:
        set_back([weechat.config_get_plugin('away_message')])


def check_timer():
    """Sets or unsets the timer
    based on whether or not the plugin is enabled"""
    global TIMER
    if TIMER:
        weechat.unhook(TIMER)
    TIMER = weechat.hook_timer(
            int(weechat.config_get_plugin('interval')) * 1000,
            0, 0, "auto_check", "")

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                  SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        for option, default_value in settings.iteritems():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)

        weechat.hook_command(SCRIPT_COMMAND,
                             SCRIPT_DESC,
                             "msg <status>",
                             "msg: set the away message\n",
                             "", "slock_away_cb", "")
        check_timer()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Hairo R. Carela <hairocr8@gmail.com>
#
# Everyone is permitted to copy and distribute verbatim or modified
# copies of this license document, and changing it is allowed as long
# as the name is changed.
#
# DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
# TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#
# 0. You just DO WHAT THE FUCK YOU WANT TO.
#
# Alternate way of text formatting, useful for relays without text formatting
# features (Glowingbear, WeechatAndroid, etc)
#
# Usage:
# /aformat *text* for bold text
# /aformat /text/ for italic text
# /aformat _text_ for underlined text
# /aformat |text| for reversed (black on white) text
#
# History:
#   2016-09-24:
#       v0.1: Initial release
#   2018-06-19:
#       v0.2: py3k-ok
#
# TODO:
# - Colors support

import sys

try:
    import weechat
    from weechat import WEECHAT_RC_OK
    import_ok = True
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

SCRIPT_NAME = "aformat"
SCRIPT_AUTHOR = "Hairo R. Carela <hairocr8@gmail.com>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "WTFPL"
SCRIPT_DESC = ("Alternate way of text formatting, see /help for instructions")

PY3 = sys.version > '3'

class format:
    # Special byte sequences, using weechat.color("stuff") had some unwanted
    # results, i'll look into it if needed. Colors are unused for now
   BOLD = '\x02'
   ITALIC = '\x1D'
   UNDERLINE = '\x1F'
   REVERSE = '\x16'
   END = '\x0F'

if PY3:
    unichr = chr
    def send(buf, text):
        weechat.command(buf, "/input send {}".format(text))
else:
    def send(buf, text):
        weechat.command(buf, "/input send {}".format(text.encode("utf-8")))

def cb_aformat_cmd(data, buf, args):
    if not PY3:
        args = args.decode("utf-8")

    # Get the indexes of the separators (*/_|) in the string
    bolds = [i for i, ltr in enumerate(args) if ltr == "*"]
    italics = [i for i, ltr in enumerate(args) if ltr == "/"]
    underlines = [i for i, ltr in enumerate(args) if ltr == "_"]
    reverses = [i for i, ltr in enumerate(args) if ltr == "|"]

    if len(bolds) != 0:
        for i, v in enumerate(bolds):
            if i%2 == 0:
                args = args[:v] + format.BOLD + args[v+1:]
            else:
                args = args[:v] + format.END + args[v+1:]

    if len(italics) != 0:
        for i, v in enumerate(italics):
            if i%2 == 0:
                args = args[:v] + format.ITALIC + args[v+1:]
            else:
                args = args[:v] + format.END + args[v+1:]

    if len(underlines) != 0:
        for i, v in enumerate(underlines):
            if i%2 == 0:
                args = args[:v] + format.UNDERLINE + args[v+1:]
            else:
                args = args[:v] + format.END + args[v+1:]

    if len(reverses) != 0:
        for i, v in enumerate(reverses):
            if i%2 == 0:
                args = args[:v] + format.REVERSE + args[v+1:]
            else:
                args = args[:v] + format.END + args[v+1:]

    send(buf, args)
    return weechat.WEECHAT_RC_OK


if import_ok and __name__ == "__main__":
    weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                     SCRIPT_LICENSE, SCRIPT_DESC, '', '')
    weechat.hook_command("aformat", "Alternate way of text formatting, useful for relays without text formatting features (Glowingbear, WeechatAndroid, etc)",
                         "text <*/_|> text <*/_|> more text",
                         "    *: bold text\n"
                         "    /: italic text\n"
                         "    _: underlined text\n"
                         "    |: reversed (black on white) text\n\n"
                         "    eg.: typing: /aformat This /must/ be the *work* of an _enemy_ |stand|\n"
                         "    will output: This {0}must{4} be the {1}work{4} of an {2}enemy{4} {3}stand{4}".format(weechat.color("italic"), weechat.color("bold"), weechat.color("underline"), weechat.color("reverse"), weechat.color("reset")),
                         "", "cb_aformat_cmd", "")



# ==================================================== #
# Script Name: queue.py	
# Script Author: walk <KingPython@gmx.com>
# Script Purpose: Command queing at its finest. Hopefully.
#
# Copyright (C) 2011  walk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Version History:
#
# 0.4.2 - Nov 22nd, 2015
#    Add saving of static queues to disk and reloading them on startup.
#    Added by Tim Kuhlman - https://github.com/tkuhlman
# 0.4.1 - Jan 20th, 2011
#    Multi-list queuing seems to work flawlessly so far. Expanded on the /help qu text.
#    Properties are fully-functional. As for loading/saving of lists, I want to hold off until
#    I get some feedback on whether or not that would be applicable. Please email me
#    at the listed address in this script and let me know if you want this feature and/or
#    any other features.
#
# 0.4.0 - Jan 16th, 2011
#    Finished adding multi-list queuing. So far, no bugs that I can tell.
#    Perhaps an ability to load/save lists for multiple uses? Now working on
#    setting individual properties per list then taking a few days off this thing.
#    ** TESTED ON WeeChat 0.3.3 and 0.3.4 **
#
# 0.3.5 - Jan 15th, 2011
#    Started to implement multiple queue lists.
#
# 0.3.5 - Jan 14th, 2011
#    Wrote Queue class and merged with code. I did this because
#    some future release will include an ability to use multiple
#    queue lists. Also fixed some bugs in parsing and executing
#    arguments with /qu. /qu del by itself raised an error, /qu del j
#    raised a valueerror, fixed. /qu add singlearg wouldn't add. Fixed.
#
# 0.3.4 - Jan 11th, 2011
#    Modified configuration code to use plugins.conf
#
# 0.3.3 - Jan 9th, 2011
#    Big code clean-up. Reduced a few lines by optimizing code.
#    Next step, convertcommand_qu dictionary to array like
#    I should have done in the first place. Could save some lines.
#
# 0.3.2 - Jan 9th, 2011
#    Added RAINBOW option (requested)
#    Set as option in configuration
#
# 0.3.1 - Jan 8th, 2011
#    Added configurations (verbose, core output only)
#
# 0.3.0 - Jan 6th, 2011
#    Worked on script for quite a while. All code is functional.
#    Continuing to test for bugs and improve code
#    Fixed indexing issue after using /qu del with remove_index function
#
# 0.2.0 - Jan 5th, 2011
#   Cleaned up temporary testing code, built upon foundation. 
#   Next to include, del function and list function
#
# 0.1.0 - Jan 4th, 2011 
#   Wrote basic outline with minimal functionality
# ==================================================== #

import os
import pickle

import_ok = True
try:
	import weechat
except ImportError:
	print("This script requires WeeChat")
	print("To obtain a copy, for free, visit http://weechat.org")
	import_ok = False


class Queue():
	""" A Queuing Class """
	def __init__(self):
		self.data = []
		self.index = len(self.data)
		self.__clearable__ = True
		self.__ldl__ = ''
		self.__locked__ = False
	
	def __iter__(self):
		for char in range(self.index):
			yield self.data[char]
	
	def __len__(self):
		return len(self.data)
	
	def add(self, queue_text):
		if self.__locked__ == False:
			self.data.append(queue_text)
			self.index = len(self.data)
	
	def remove(self, info):
		if self.__locked__ == False:
			tmp = ''
			if info > 0:
				tmp = str(self.data[info-1])
				del self.data[info-1]
				self.__ldl__ = tmp
				self.index = len(self.data)
			elif info == 0:
				tmp = str(self.data[info])
				del self.data[info] 
				self.__ldl__ = tmp
				self.index = len(self.data)
		
		return self.__ldl__
	
	def viewqueue(self):
		list = ''
		if not len(self.data) == 0:
			for each in range(len(self.data)):
				list+=str(each+1) + ". " + str(self.data[each]) + "\n"
		else:
			list='Nothing in queue'
		
		return list.strip("\n")
	
	def clearqueue(self):
		if self.__clearable__ == True:
			self.data = []
			self.index = 0

	def isClear(self, cOpt):
		if not cOpt in (True, False):
			return
		
		self.__clearable__ = cOpt
	
	def isLocked(self, lockOpt):
		if not lockOpt in (True, False):
			return
		
		self.__locked__ = lockOpt
	
	def isEmpty(self):
		if len(self.data) == 0:
			return True
		else:
			return False


SCRIPT_NAME		= "queue"
SCRIPT_AUTHOR	= "walk"
SCRIPT_VERSION	= "0.4.2"
SCRIPT_LICENSE	= "GPL3"
SCRIPT_DESC		= "Command queuing"

COMM_CMD		= "qu"
COMM_DESC		= "Queuing commands in WeeChat"
COMM_ARGS		= "[add [command] | del [index] | new [list] | dellist [list] | set [property] [on|off] |list | clear | exec | listview]"
COMM_ARGS_DESC		= "Examples: \n\
   /qu add /msg chanserv op #foo bar \n\
   /qu del 1 \n\
   /qu new weechat \n\
       - Use the 'new' argument to switch to already defined lists as well. \n\
   /qu dellist weechat \n\
   /qu list - List commands in current list \n\
   /qu list weechat - With optional parameter, you can choose to list the commands of a specified list. \n\
   /qu clear - Clear current list.. add a listname to clear a specified list. \n\
   /qu exec - Execute the commands of the current list.. you can also specify a list here as well. \n\
   /qu listview - Outputs the names of all your lists. \n\
   /qu save - Save static lists to disk \n\
   /qu set static on - Sets static property to ON for current list. This means that when executed, the list WILL NOT clear. The clear command will not work either.\n \
   \n\
   PROPERTIES (for set command):\n \
   static - prevents a list from clearing manually or automatically but can still add and del commands.\n \
   lock - prevents the user from adding/deleting entries to a list. Can be combined with static."
COMM_COMPL		= "add|del|list|exec|new|listview|dellist"

COMMAND_QU = {'default': Queue()}
CURR_LIST = 'default'

def __config__():
	""" Configuration initialization """
	
	if not weechat.config_get_plugin("core_output_only") in ("yes", "no"):
		weechat.config_set_plugin("core_output_only", "yes")
	if not weechat.config_get_plugin("rainbow_allow") in ("yes", "no"):
		weechat.config_set_plugin("rainbow_allow", "no")
	if not weechat.config_get_plugin("verbose") in ("yes", "no"):
		weechat.config_set_plugin("verbose", "yes")
	
	load()
	return weechat.WEECHAT_RC_OK

def load():
    """ Load saved queues from pickle. """
    global COMMAND_QU
    pickle_path = os.path.join(weechat.info_get("weechat_dir", ""), 'queue.pickle')
    if os.path.exists(pickle_path):
        with open(pickle_path, 'r') as qu_pickle:
            COMMAND_QU = pickle.load(qu_pickle)

    if 'default' not in COMMAND_QU:
        COMMAND_QU['default'] = Queue()

def rainbow(data):
	""" Not my favorite option but a requested one """

	colors='red yellow green blue magenta'
	c=colors.split()
	count=0
	colorHolder=''
	
	for each in data:
		if count > 4: count = 0
		if not each == " ":
			colorHolder+=weechat.color(c[count])+each
			count += 1
		else:
			colorHolder+=" "
	
	return str(colorHolder)

def prntcore(data, essential=0, rb=0):
	""" Built more on weechat.prnt """

	if weechat.config_get_plugin("verbose") == 'yes' or essential==1:
		if weechat.config_get_plugin("core_output_only") =='yes':
			buffer = ''
		else:
			buffer = weechat.current_buffer()
		if rb == 0:
			weechat.prnt(buffer, data)
		else:
			weechat.prnt(buffer, rainbow(data))
	return weechat.WEECHAT_RC_OK

def save():
    """ Save to disk all static lists as a pickle. """
    global COMMAND_QU
    pickle_path = os.path.join(weechat.info_get("weechat_dir", ""), 'queue.pickle')
    to_save = {}
    for name, qu in COMMAND_QU.iteritems():
        if not qu.__clearable__:  # Note isClear method doesn't show status it sets it
            to_save[name] = qu

    with open(pickle_path, 'w') as qu_pickle:
        pickle.dump(to_save, qu_pickle, pickle.HIGHEST_PROTOCOL)

def rejoin(data, delimiter=' '):
	""" Rejoins a split string """
	tmpString = ''
	for each in data:
		tmpString+=each+delimiter
	
	tmpString = tmpString.strip()
	return tmpString
	
def qu_cb(data, buffer, args):
	""" Process hook_command info """
	
	global CURR_LIST, COMMAND_QU
	if weechat.config_get_plugin('rainbow_allow') == 'no':
		rainbowit=0
	else:
		rainbowit=1

        if args == "":
                return weechat.WEECHAT_RC_OK

	argv = args.split()
	arglist = ['add', 'del', 'new', 'dellist', 'list', 'clear', 'exec', 'listview', 'save', 'set']
	
	if not argv[0] in arglist:
		prntcore('[ queue -> not a valid argument: {0}'.format(argv[0]), rb=rainbowit)
		return weechat.WEECHAT_RC_OK
	
	if argv[0].lower() == "add" and len(argv) > 1:
		if not COMMAND_QU[CURR_LIST].__locked__ == True:
			COMMAND_QU[CURR_LIST].add(rejoin(argv[1:]))
			prntcore("[ queue added -> "+str(rejoin(argv[1:])) + " ]", rb=rainbowit)
		else:
			prntcore("[ queue -> the lock property is enabled for this list ({0}). please disable it before adding/deleting. ]", rb=rainbowit)
	
	elif argv[0].lower() == "del" and len(argv) > 1:
		if not COMMAND_QU[CURR_LIST].__locked__ == True:
			try:
				rmd = COMMAND_QU[CURR_LIST].remove(int(argv[1]))
				prntcore("[ queue -> deleted: ({0}) {1} ]".format(argv[1],rmd), rb=rainbowit)
			except (IndexError, ValueError):
				prntcore("[ queue -> invalid reference. please check /qu list and try again. ]", rb=rainbowit)
		else:
			prntcore("[ queue -> the lock property is enabled for this list ({0}). please disable it before adding/deleting. ]".format(CURR_LIST), rb=rainbowit)
		
	elif argv[0].lower() == "clear":
		this_list = None
		if len(argv) > 1 and argv[1].lower() in COMMAND_QU.keys():
			this_list = CURR_LIST
			CURR_LIST = argv[1].lower()
		
		if COMMAND_QU[CURR_LIST].__clearable__ == True:
			if not COMMAND_QU[CURR_LIST].isEmpty():
				COMMAND_QU[CURR_LIST].clearqueue()
				prntcore('[ queue -> command queue list cleared. ]', rb=rainbowit)
			else:
				prntcore('[ queue -> command queue already empty. ]', rb=rainbowit)
		else:
			prntcore('[ queue -> please turn off the static property to clear the {0} list. ]'.format(CURR_LIST), rb=rainbowit)
		
		if not this_list == None:
			CURR_LIST = this_list
			this_list = None
		
	elif argv[0].lower() == "list":
		this_list = None
		if len(argv) > 1 and argv[1].lower() in COMMAND_QU.keys():
			this_list = CURR_LIST
			CURR_LIST = argv[1].lower()
		
		qHeader = '[ COMMAND QUEUE: {0} ]'.format(CURR_LIST)
		prntcore(" ", 1)
		prntcore("-"*len(qHeader), 1, rb=rainbowit)
		prntcore(qHeader, 1, rainbowit)
		prntcore("-"*len(qHeader), 1, rainbowit)
		prntcore(COMMAND_QU[CURR_LIST].viewqueue(), 1, rb=rainbowit)
		
		if not this_list == None:
			CURR_LIST = this_list
			this_list = None
		
	elif argv[0].lower() == "exec":
		
		this_list = None
		if len(argv) > 1 and argv[1].lower() in COMMAND_QU.keys():
			this_list = CURR_LIST
			CURR_LIST = argv[1].lower()
		
		if len(COMMAND_QU[CURR_LIST]) > 0:

			for each in COMMAND_QU[CURR_LIST]:
				weechat.command(buffer, each)
			COMMAND_QU[CURR_LIST].clearqueue()
			if COMMAND_QU[CURR_LIST].__clearable__ == True:
				prntcore('[ queue -> finished executing list: {0}. command list cleared. ]'.format(CURR_LIST), rb=rainbowit)
			else:
				prntcore('[ queue -> finished executing list: {0} ]'.format(CURR_LIST), rb=rainbowit)
		else:
			prntcore("[ queue -> nothing to execute. please add to the queue using /qu add <command>", rb=rainbowit)
		
		if not this_list == None:
			CURR_LIST = this_list
			this_list = None
	
	elif argv[0].lower() == "new" and len(args.split()) > 1:
		if argv[1].lower() in COMMAND_QU.keys():
			CURR_LIST = argv[1].lower()
			prntcore("[ queue -> switched queue list to: {0}".format(CURR_LIST), rb=rainbowit)
		else:
			COMMAND_QU[argv[1].lower()] = Queue()
			CURR_LIST = argv[1].lower()
			prntcore("[ queue -> created new list. current list is: {0}".format(CURR_LIST), rb=rainbowit)
	
	elif argv[0].lower() == "listview":
		qHeader = 'QUEUE LISTS'
		listCount = 1
		
		prntcore(' ', 1)
		prntcore('-'*len(qHeader), 1, rb=rainbowit)
		prntcore(qHeader, 1, rb=rainbowit)
		prntcore('-'*len(qHeader), 1, rb=rainbowit)
		
		for each in COMMAND_QU.keys():
			prntcore(str(listCount) + ". " + str(each), 1, rb=rainbowit)
			listCount += 1
	
	elif argv[0].lower() == "dellist" and len(args.split()) > 1:
		if not argv[1].lower() in COMMAND_QU.keys():
			prntcore('[ queue -> {0} is not a list. ]'.format(argv[1].lower()), rb=rainbowit)
		elif argv[1].lower() == 'default':
			prntcore('[ queue -> cannot delete the default list. ]', rb=rainbowit)
		else:
			if argv[1].lower() == CURR_LIST:
				CURR_LIST = 'default'
			del COMMAND_QU[argv[1].lower()]
			prntcore('[ queue -> {0} successfully deleted.'.format(argv[1].lower()), rb=rainbowit)
	
	elif argv[0].lower() == "save":
		save()
	elif argv[0].lower() == "set" and len(argv) == 4:
		setargs = args.split()
		list_name = setargs[1].lower()
		set_prop = setargs[2].lower()
		toggle = setargs[3].lower()
		properties = ['static', 'lock']

		if not list_name in COMMAND_QU.keys():
			prntcore('[ queue -> list must be created before you can set properties ]', rb=rainbowit)
		elif not set_prop in properties:
			prntcore('[ queue -> invalid property. please try again. ]', rb=rainbowit)
		elif not toggle in ('on', 'off'):
			prntcore('[ queue -> only valid options for a property are ON or OFF ]', rb=rainbowit)
		else:
			if set_prop == 'static':
				if toggle=='on':
					COMMAND_QU[list_name].isClear(False)
					prntcore('[ queue -> static property toggled on for: {0} ]'.format(list_name), rb=rainbowit)
					save()
				else:
					COMMAND_QU[list_name].isClear(True)
					prntcore('[ queue -> static property toggled off for: {0} ]'.format(list_name), rb=rainbowit)
					save()
			
			elif set_prop == 'lock':
				if toggle=='on':
					COMMAND_QU[list_name].isLocked(True)
					prntcore('[ queue -> lock property toggled on for: {0} ]'.format(list_name), rb=rainbowit)
				else:
					COMMAND_QU[list_name].isLocked(False)
					prntcore('[ queue -> lock property toggled off for: {0} ]'.format(list_name), rb=rainbowit)
	
	return weechat.WEECHAT_RC_OK

if import_ok and weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
	weechat.hook_command(COMM_CMD, COMM_DESC, COMM_ARGS, COMM_ARGS_DESC, COMM_COMPL, "qu_cb", "")
	__config__()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012-2013 by nils_2 <weechatter@arcor.de>
#
# cylce to currently used server if you are using merged server buffer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2013-01-25: nils_2, (freenode.#weechat)
#       0.4 : make script compatible with Python 3.x
#
# 2012-01-28: nils_2,(freenode.#weechat)
#       0.3 : adapted to bugfix #31158 and new signal hook_signal("window_switch")
#
# 2012-01-27: nils_2,(freenode.#weechat)
#       0.2 : fix: bug with split windows removed (reported by meingtsla)
#
# 2012-01-22: nils_2,(freenode.#weechat)
#       0.1 : initial release
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat,re

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "server_autoswitch"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "0.4"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "cycle to currently used server if you are using merged server buffer"

look_server = ""

def window_switch_cb(data, signal, signal_data):
    bufpointer = weechat.window_get_pointer(signal_data,"buffer")
    buffer_switch_cb(data,signal,bufpointer)
    return weechat.WEECHAT_RC_OK
def buffer_switch_cb(data, signal, signal_data):
    global look_server
    look_server = ""
    look_server = weechat.config_string(weechat.config_get("irc.look.server_buffer"))
    if  look_server == "independent":                                                   # server buffer independent?
        return weechat.WEECHAT_RC_OK                                                    # better remove script, you don't need it.

    if weechat.buffer_get_string(signal_data,'name') != 'weechat':                      # not weechat core buffer
        if (weechat.buffer_get_string(signal_data,'localvar_type') == '') or (weechat.buffer_get_string(signal_data,'localvar_type') == 'server'):
            return weechat.WEECHAT_RC_OK
    elif weechat.buffer_get_string(signal_data,'name') == 'weechat':
        return weechat.WEECHAT_RC_OK

    # buffer is channel or private?
    if (weechat.buffer_get_string(signal_data,'localvar_type') == 'channel') or (weechat.buffer_get_string(signal_data,'localvar_type') == 'private'):
        bufpointer = weechat.window_get_pointer(weechat.current_window(),"buffer")
        servername_from_current_buffer = weechat.buffer_get_string(bufpointer, 'localvar_server')
        name = weechat.buffer_get_string(bufpointer, 'name')
        server_switch(signal_data,servername_from_current_buffer,name)
    return weechat.WEECHAT_RC_OK

def server_switch(signal_data,servername_from_current_buffer,name):
    global look_server
    SERVER = {}

    bufpointer = weechat.window_get_pointer(weechat.current_window(),"buffer")
    servername_current_buffer = servername_from_current_buffer
    if look_server == "merge_with_core":                                                # merge_with_core
        SERVER["weechat"] = "core.weechat"

# get ALL server buffers and save them
    infolist = weechat.infolist_get("buffer","","*server.*")                            # we are only interest in server-buffers
    while weechat.infolist_next(infolist):
        bufpointer = weechat.infolist_pointer(infolist,"pointer")
        server = weechat.infolist_string(infolist, "name")                              # full servername (server.<servername>)
        servername = weechat.infolist_string(infolist, "short_name")                    # get servername from server (without prefix "server")
        active = weechat.infolist_integer(infolist,"active")
        SERVER[servername] = server
        if (active == 1) and (servername_current_buffer != servername):                 # buffer active but not correct server buffer?
            weechat.command(bufpointer,"/input switch_active_buffer")                   # switch server buffer
    weechat.infolist_free(infolist)                                                     # do not forget to free infolist!

# switch though all server and stop at server from current buffer
    i = 0
    while i <= len(SERVER):
        for servername,full_name in list(SERVER.items()):
            bufpointer = weechat.buffer_search("irc","%s" % full_name)                  # search pointer from server buffer
            if bufpointer == "":                                                        # core buffer
                if weechat.buffer_get_integer(weechat.buffer_search_main(),'active') == 1:
                    weechat.command(weechat.buffer_search_main(),"/input switch_active_buffer")
            else:                                                                       # server buffer!
                if (servername != servername_current_buffer) and (weechat.buffer_get_integer(bufpointer,'active') == 1):
                    weechat.command(bufpointer,"/input switch_active_buffer")
                elif (servername == servername_current_buffer) and (weechat.buffer_get_integer(bufpointer,'active') == 1):
                    i = len(SERVER)
                    break
        i += 1
# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get("version_number", "") or 0
        if int(version) >= 0x00030600:
            weechat.hook_signal("buffer_switch","buffer_switch_cb","")
            weechat.hook_signal("window_switch","window_switch_cb","")
        else:
            weechat.prnt("","%s%s %s" % (weechat.prefix("error"),SCRIPT_NAME,": needs version 0.3.6 or higher"))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 by nils_2 <weechatter@arcor.de>
#
# mark buffers as read if there is no new message in a specific time range
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# idea by bascht
#
# 2015-01-25: nils_2, (freenode.#weechat)
#       1.0 : initial release

try:
    import weechat
    import time

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "automarkbuffer"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "1.0"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "mark buffers as read if there is no new message in a specific time range"
TIMER           = None
WEECHAT_VERSION = ""
whitelist       = []

OPTIONS         = { 'whitelist'                         : ('','comma separated list of buffer to ignore, e.g. freenode.#weechat,freenode.#weechat-de (check name of buffer with : /buffer localvar)'),
                    'time'                              : ('3600','time in seconds to mark buffer as read, if there are no new messages'),
                    'interval'                          : ('60','how often in seconds to check for messages'),
                    'clear'                             : ('all','hotlist = remove buffer from hotlist, unread = set unread marker, all = hotlist & unread'),
                    'ignore_hidden'                     : ('on','hidden messages will be ignored (for example "irc_smart_filter" ones)'),
                    'ignore_query'                      : ('on','query buffer(s) will be ignored'),
                   }

def check_buffer_timer_cb(data, remaining_calls):
    global WEECHAT_VERSION,whitelist

    # search for buffers in hotlist
    ptr_infolist = weechat.infolist_get("hotlist", "", "")
    while weechat.infolist_next(ptr_infolist):
        ptr_buffer = weechat.infolist_pointer(ptr_infolist, "buffer_pointer")
        localvar_name = weechat.buffer_get_string(ptr_buffer, 'localvar_name')
        # buffer in whitelist? go to next buffer
        buf_type = weechat.buffer_get_string(ptr_buffer,'localvar_type')
        # buffer is a query buffer?
        if OPTIONS['ignore_query'].lower() == 'on' and buf_type == 'private':
            continue
        # buffer in whitelist?
        if localvar_name in whitelist:
            continue
        if ptr_buffer:
            if get_time_from_line(ptr_buffer):
                if OPTIONS['clear'].lower() == 'hotlist' or OPTIONS['clear'].lower() == 'all':
                    weechat.buffer_set(ptr_buffer, "hotlist", '-1')
                if OPTIONS['clear'].lower() == 'unread' or OPTIONS['clear'].lower() == 'all':
                    weechat.command(ptr_buffer,"/input set_unread_current_buffer")

    weechat.infolist_free(ptr_infolist)
    return weechat.WEECHAT_RC_OK

def get_time_from_line(ptr_buffer):
    lines = weechat.hdata_pointer(weechat.hdata_get('buffer'), ptr_buffer, 'own_lines')

    if lines:
        line = weechat.hdata_pointer(weechat.hdata_get('lines'), lines, 'last_line')
        last_read_line = weechat.hdata_pointer(weechat.hdata_get('lines'), lines, 'last_read_line')
        # last line already read?
        while line != last_read_line:
            hdata_line = weechat.hdata_get('line')
            hdata_line_data = weechat.hdata_get('line_data')

            data = weechat.hdata_pointer(hdata_line, line, 'data')

            date_last_line = weechat.hdata_time(hdata_line_data, data, 'date')
            displayed = weechat.hdata_char(hdata_line_data, data, 'displayed')
            # message hidden?
            if not displayed and OPTIONS['ignore_hidden'].lower() == 'on':
                prev_line = weechat.hdata_pointer(hdata_line, line, 'prev_line')
                line = prev_line
                continue

            # buffer empty?
            if not date_last_line:
                return 0

            get_current_ticks = time.time()

            time_gone = get_current_ticks - date_last_line
            if int(OPTIONS['time']) < time_gone:
                return 1
            else:
                return 0
    return 0
# ================================[ weechat options and description ]===============================
def set_timer():
    global TIMER
    if TIMER:
        weechat.unhook(TIMER)
    if int(OPTIONS['interval']) == 0:
        return
    TIMER = weechat.hook_timer(int(OPTIONS['interval']) * 1000,
            0, 0, 'check_buffer_timer_cb', '')

def init_options():
    for option,value in OPTIONS.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = weechat.config_get_plugin(option)
        weechat.config_set_desc_plugin(option, "%s (default: '%s')" % (value[1], value[0]))

def toggle_refresh(pointer, name, value):
    global OPTIONS,whitelist
    option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]        # get optionname
    OPTIONS[option] = value                                               # save new value
    if name.endswith('.interval'):                                        # timer value changed?
        set_timer()

    if name.endswith('.whitelist'):                                       # whitelist changed?
        whitelist = OPTIONS['whitelist'].split(',')
    return weechat.WEECHAT_RC_OK
# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        WEECHAT_VERSION = weechat.info_get("version_number", "") or 0

        if int(WEECHAT_VERSION) >= 0x01000000:
            weechat.hook_config( 'plugins.var.python.' + SCRIPT_NAME + '.*', 'toggle_refresh', '' )
            init_options()
            set_timer()
            whitelist = OPTIONS['whitelist'].split(',')
        else:
            weechat.prnt('','%s%s %s' % (weechat.prefix('error'),SCRIPT_NAME,': needs version 1.0 or higher'))



# -*- coding: utf-8 -*-
###
# Copyright (c) 2010 by xt <xt@bash.no>
# License: GPL3
#
#
#
#   Usage scenarios:
#
#   * Remote control weechat from a jabber account (from phone for example)
#       Requires you to send commands starting with a /
#   * Reply to messages sent via away_action
#       Requires away_action.py installed, will send input to the last buffer away_action recieved a
#       message from
#       
#
#   History:
#   2010-11-08:
#   version 0.2: check that message is from remote side
#   2010-11-04
#   version 0.1: initial release
#
###

SCRIPT_NAME    = "msg_command"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Run chat recieved in a buffer as commands"

### Default Settings ###
settings = {
        'buffer': 'jabber.gtalk.otheraccount@otherserver.com', # Buffer to listen for commands
}

buffer_hooked_pointer = ''
hook = ''

try:
    import weechat
    w = weechat
    WEECHAT_RC_OK = weechat.WEECHAT_RC_OK
    import_ok = True
except:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

def msg_command_cb(data, buffer, time, tags, display, hilight, prefix, msg):

    if msg.startswith('/'):
        w.command('', msg)
    else:
        buffer_name = w.info_get('away_action_buffer', '')
        plugin = buffer_name.split('.')[0]
        buffer_name = '.'.join(buffer_name.split('.')[1:])
        buffer = w.buffer_search(plugin, buffer_name)
        if buffer:
            # Check if message is from remote and not a local
            if prefix in w.config_get_plugin('buffer'):
                w.command(buffer, msg)
    return WEECHAT_RC_OK

def hook_it(buffer_pointer):
    ''' Check if we need new hook, remove previous hook if exists '''

    global buffer_hooked_pointer, hook
    if buffer_pointer != buffer_hooked_pointer:
        w.unhook(hook)
        buffer_hooked_pointer = buffer_pointer
        hook = w.hook_print(buffer_pointer, '', '', 1, 'msg_command_cb', '')

def msg_command_conf_update(*args):
    if w.config_get_plugin('buffer'):
        buffer = w.buffer_search('', w.config_get_plugin('buffer'))
        hook_it(buffer)
    return WEECHAT_RC_OK


def msg_command_buffer_opened_cb(data, signal, signal_data):

    buffer = signal_data
    buffer_name = w.buffer_get_string(buffer, "name")
    if buffer_name == w.config_get_plugin('buffer'):
        hook_it(buffer)

    return WEECHAT_RC_OK

if __name__ == '__main__' and import_ok and \
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC,
        '', ''):


    for opt, val in settings.iteritems():
        if not weechat.config_is_set_plugin(opt):
            weechat.config_set_plugin(opt, val)

    weechat.hook_signal('buffer_opened', 'msg_command_buffer_opened_cb', '')
    weechat.hook_config('plugins.var.python.%s' %SCRIPT_NAME, 'msg_command_conf_update', '')
    msg_command_conf_update() # To init hook


# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# Author: tobypadilla <tobypadilla AT gmail DOT com>
# Homepage: http://github.com/tobypadilla/gnotify
# Version: 0.2
#
# gnotify requires Growl Python Bindings
# See here: http://growl.info/documentation/developer/python-support.php
# Requires Weechat 0.3.0
# Released under GNU GPL v2
#
# Copy weechaticn.png to /usr/local/share/pixmaps/ or change the path below
#
# gnotify is derived from notify http://www.weechat.org/files/scripts/notify.py
# Original author: lavaramano <lavaramano AT gmail DOT com>

import weechat, Growl, string

weechat.register("gnotify", "tobypadilla", "0.2", "GPL", "gnotify: Growl notifications for Weechat", "", "")

# script options
settings = {
    "show_hilights"             : "on",
    "show_priv_msg"             : "on",
    "sticky"                    : "on",
    "icon"                      : "/usr/local/share/pixmaps/weechaticn.png",
}

# Setup Growl Notification Class
class gNotifier(Growl.GrowlNotifier):
   applicationName = 'gnotify'
   notifications = ['highlight']
   # don't have/want the weechat icon? use ichat instead
   # applicationIcon = Growl.Image.imageWithIconForApplication("iChat")
   applicationIcon = Growl.Image.imageFromPath(weechat.config_get_plugin('icon'))

# Init everything
for option, default_value in settings.items():
    if weechat.config_get_plugin(option) == "":
        weechat.config_set_plugin(option, default_value)

# Hook commands to setup stickynote command
hook = weechat.hook_command("stickynote", "Set stickyness of gnotify Growl notifications",
    "[on|off|toggle]",
    "'on' makes sticky, 'off' makes not sticky, 'toggle' toggles stickyness",
    "on || off || toggle",
    "stickynote_cmd",
    "")

def stickynote_cmd(data, buffer, args):
    isSticky = weechat.config_get_plugin('sticky')
    if (args == 'toggle'):
        isSticky = ('on' if isSticky == 'off' else 'off')
    elif (args == 'on' or args == 'off'):
        isSticky = args
    else:
        weechat.prnt("","Invalid stickynote option: "+args)
    weechat.config_set_plugin('sticky',isSticky)
    weechat.prnt("","Growl notification stickyness is "+isSticky)
    return weechat.WEECHAT_RC_OK

# Hook privmsg/hilights
weechat.hook_signal("weechat_pv", "notify_show_priv", "")
weechat.hook_signal("weechat_highlight", "notify_show_hi", "")

# Functions
def notify_show_hi( data, signal, message ):
    """Sends highlight message to be printed on notification"""
    if weechat.config_get_plugin('show_hilights') == "on":
        show_notification("Weechat",  message)
    return weechat.WEECHAT_RC_OK

def notify_show_priv( data, signal, message ):
    """Sends private message to be printed on notification"""
    if weechat.config_get_plugin('show_priv_msg') == "on":
        show_notification("Weechat Private Message",  message)
    return weechat.WEECHAT_RC_OK

def show_notification(title,message):
    growl = gNotifier()
    growl.register()

    isSticky = (True if weechat.config_get_plugin('sticky') == "on" else False)
    growl.notify('highlight', title, message, "", isSticky)

